@@ -37,6 +37,7 @@ struct HashList {
 	char **MyKeys;         /**< this keeps the members for a call of GetHashKeys */
 	HashFunc Algorithm;    /**< should we use an alternating algorithm to calc the hash values? */
 	long nMembersUsed;     /**< how many pointers inside of the array are used? */
+	long nLookupTableItems; /**< how many items of the lookup table are used? */
 	long MemberSize;       /**< how big is Members and LookupTable? */
 	long tainted;          /**< if 0, we're hashed, else s.b. else sorted us in his own way. */
 	long uniq;             /**< are the keys going to be uniq? */
@@ -69,7 +70,7 @@ int PrintHash(HashList *Hash, TransitionFunc Trans, PrintHashDataFunc PrintEntry
 	if (Hash == NULL)
 		return 0;
 
-	for (i=0; i < Hash->nMembersUsed; i++) {
+	for (i=0; i < Hash->nLookupTableItems; i++) {
 		if (i==0) {
 			Previous = NULL;
 		}
@@ -116,11 +117,11 @@ int dbg_PrintHash(HashList *Hash, PrintHashContent First, PrintHashContent Secon
 	if (Hash->MyKeys != NULL)
 		free (Hash->MyKeys);
 
-	Hash->MyKeys = (char**) malloc(sizeof(char*) * Hash->nMembersUsed);
+	Hash->MyKeys = (char**) malloc(sizeof(char*) * Hash->nLookupTableItems);
 #ifdef DEBUG
 	printf("----------------------------------\n");
 #endif
-	for (i=0; i < Hash->nMembersUsed; i++) {
+	for (i=0; i < Hash->nLookupTableItems; i++) {
 		
 		if (Hash->LookupTable[i] == NULL)
 		{
@@ -179,7 +180,7 @@ HashList *NewHash(int Uniq, HashFunc F)
 int GetCount(HashList *Hash)
 {
 	if(Hash==NULL) return 0;
-	return Hash->nMembersUsed;
+	return Hash->nLookupTableItems;
 }
 
 
@@ -219,6 +220,7 @@ void DeleteHash(HashList **Hash)
 	FreeMe = *Hash;
 	if (FreeMe == NULL)
 		return;
+	/* even if there are sparse members already deleted... */
 	for (i=0; i < FreeMe->nMembersUsed; i++)
 	{
 		/** get rid of our payload */
@@ -258,6 +260,16 @@ static void IncreaseHashSize(HashList *Hash)
 	if (Hash == NULL)
 		return ;
 
+	/** If we grew to much, this might be the place to rehash and shrink again.
+	if ((Hash->NMembersUsed > Hash->nLookupTableItems) && 
+	    ((Hash->NMembersUsed - Hash->nLookupTableItems) > 
+	     (Hash->nLookupTableItems / 10)))
+	{
+
+
+	}
+	*/
+
 	/** double our payload area */
 	NewPayloadArea = (Payload**) malloc(sizeof(Payload*) * Hash->MemberSize * 2);
 	memset(&NewPayloadArea[Hash->MemberSize], 0, sizeof(Payload*) * Hash->MemberSize);
@@ -317,11 +329,11 @@ static void InsertHashItem(HashList *Hash,
 	/** our payload is queued at the end... */
 	NewHashKey->Position = Hash->nMembersUsed;
 	/** but if we should be sorted into a specific place... */
-	if ((Hash->nMembersUsed != 0) && 
-	    (HashPos != Hash->nMembersUsed) ) {
+	if ((Hash->nLookupTableItems != 0) && 
+	    (HashPos != Hash->nLookupTableItems) ) {
 		long ItemsAfter;
 
-		ItemsAfter = Hash->nMembersUsed - HashPos;
+		ItemsAfter = Hash->nLookupTableItems - HashPos;
 		/** make space were we can fill us in */
 		if (ItemsAfter > 0)
 		{
@@ -334,6 +346,7 @@ static void InsertHashItem(HashList *Hash,
 	Hash->Members[Hash->nMembersUsed] = NewPayloadItem;
 	Hash->LookupTable[HashPos] = NewHashKey;
 	Hash->nMembersUsed++;
+	Hash->nLookupTableItems++;
 }
 
 /**
@@ -350,7 +363,7 @@ static long FindInTaintedHash(HashList *Hash, long HashBinKey)
 	if (Hash == NULL)
 		return 0;
 
-	for (SearchPos = 0; SearchPos < Hash->nMembersUsed; SearchPos ++) {
+	for (SearchPos = 0; SearchPos < Hash->nLookupTableItems; SearchPos ++) {
 		if (Hash->LookupTable[SearchPos]->Key == HashBinKey){
 			return SearchPos;
 		}
@@ -375,10 +388,10 @@ static long FindInHash(HashList *Hash, long HashBinKey)
 	if (Hash->tainted)
 		return FindInTaintedHash(Hash, HashBinKey);
 
-	SearchPos = Hash->nMembersUsed / 2;
+	SearchPos = Hash->nLookupTableItems / 2;
 	StepWidth = SearchPos / 2;
 	while ((SearchPos > 0) && 
-	       (SearchPos < Hash->nMembersUsed)) 
+	       (SearchPos < Hash->nLookupTableItems)) 
 	{
 		/** Did we find it? */
 		if (Hash->LookupTable[SearchPos]->Key == HashBinKey){
@@ -400,7 +413,7 @@ static long FindInHash(HashList *Hash, long HashBinKey)
 				SearchPos --;
 			}
 			else {
-				if ((SearchPos + 1 < Hash->nMembersUsed) && 
+				if ((SearchPos + 1 < Hash->nLookupTableItems) && 
 				    (Hash->LookupTable[SearchPos + 1]->Key > HashBinKey))
 					return SearchPos;
 				SearchPos ++;
@@ -515,7 +528,7 @@ int GetHash(HashList *Hash, const char *HKey, long HKLen, void **Data)
 	HashBinKey = CalcHashKey(Hash, HKey, HKLen);
 	HashAt = FindInHash(Hash, HashBinKey);
 	if ((HashAt < 0) || /**< Not found at the lower edge? */
-	    (HashAt >= Hash->nMembersUsed) || /**< Not found at the upper edge? */
+	    (HashAt >= Hash->nLookupTableItems) || /**< Not found at the upper edge? */
 	    (Hash->LookupTable[HashAt]->Key != HashBinKey)) { /**< somewhere inbetween but no match? */
 		*Data = NULL;
 		return 0;
@@ -549,13 +562,13 @@ int GetHashKeys(HashList *Hash, char ***List)
 	if (Hash->MyKeys != NULL)
 		free (Hash->MyKeys);
 
-	Hash->MyKeys = (char**) malloc(sizeof(char*) * Hash->nMembersUsed);
-	for (i=0; i < Hash->nMembersUsed; i++) {
+	Hash->MyKeys = (char**) malloc(sizeof(char*) * Hash->nLookupTableItems);
+	for (i=0; i < Hash->nLookupTableItems; i++) {
 	
 		Hash->MyKeys[i] = Hash->LookupTable[i]->HashKey;
 	}
 	*List = (char**)Hash->MyKeys;
-	return Hash->nMembersUsed;
+	return Hash->nLookupTableItems;
 }
 
 /**
@@ -576,7 +589,7 @@ HashPos *GetNewHashPos(HashList *Hash, int StepWidth)
 	else
 		Ret->StepWidth = 1;
 	if (Ret->StepWidth <  0) {
-		Ret->Position = Hash->nMembersUsed - 1;
+		Ret->Position = Hash->nLookupTableItems - 1;
 	}
 	else {
 		Ret->Position = 0;
@@ -584,6 +597,94 @@ HashPos *GetNewHashPos(HashList *Hash, int StepWidth)
 	return Ret;
 }
 
+/**
+ * @brief Set iterator object to point to key. If not found, don't change iterator
+ * @param Hash the list we reference
+ * @param HKey key to search for
+ * @param HKLen length of key
+ * @param At HashPos to update
+ * \returns 0 if not found
+ */
+int GetHashPosFromKey(HashList *Hash, const char *HKey, long HKLen, HashPos *At)
+{
+	long HashBinKey;
+	long HashAt;
+
+	if (Hash == NULL)
+		return 0;
+
+	if (HKLen <= 0) {
+		return  0;
+	}
+	/** first, find out were we could be... */
+	HashBinKey = CalcHashKey(Hash, HKey, HKLen);
+	HashAt = FindInHash(Hash, HashBinKey);
+	if ((HashAt < 0) || /**< Not found at the lower edge? */
+	    (HashAt >= Hash->nLookupTableItems) || /**< Not found at the upper edge? */
+	    (Hash->LookupTable[HashAt]->Key != HashBinKey)) { /**< somewhere inbetween but no match? */
+		return 0;
+	}
+	/** GOTCHA! */
+	At->Position = Hash->LookupTable[HashAt]->Position;
+	return 1;
+}
+
+/**
+ * @brief Delete from the Hash the entry at Position
+ * @param Hash the list we reference
+ * @param At the position within the Hash
+ * \returns 0 if not found
+ */
+int DeleteEntryFromHash(HashList *Hash, HashPos *At)
+{
+	Payload *FreeMe;
+	if (Hash == NULL)
+		return 0;
+
+	/* if lockable, lock here */
+	if ((Hash == NULL) || 
+	    (At->Position >= Hash->nLookupTableItems) || 
+	    (At->Position < 0) ||
+	    (At->Position > Hash->nLookupTableItems))
+	{
+		/* unlock... */
+		return 0;
+	}
+
+	FreeMe = Hash->Members[Hash->LookupTable[At->Position]->Position];
+	Hash->Members[Hash->LookupTable[At->Position]->Position] = NULL;
+
+
+	/** delete our hashing data */
+	if (Hash->LookupTable[At->Position] != NULL)
+	{
+		free(Hash->LookupTable[At->Position]->HashKey);
+		free(Hash->LookupTable[At->Position]);
+		if (At->Position < Hash->nLookupTableItems)
+		{
+			memmove(&Hash->LookupTable[At->Position],
+				&Hash->LookupTable[At->Position + 1],
+				(Hash->nLookupTableItems - At->Position - 1) * 
+				sizeof(HashKey*));
+
+			Hash->LookupTable[Hash->nLookupTableItems - 1] = NULL;
+		}
+		else 
+			Hash->LookupTable[At->Position] = NULL;
+		Hash->nLookupTableItems--;
+	}
+	/* unlock... */
+
+
+	/** get rid of our payload */
+	if (FreeMe != NULL)
+	{
+		DeleteHashPayload(FreeMe);
+		free(FreeMe);
+	}
+	return 1;
+}
+
 /**
  * @brief retrieve the counter from the itteratoor
  * @param the Iterator to analyze
@@ -592,9 +693,9 @@ HashPos *GetNewHashPos(HashList *Hash, int StepWidth)
 int GetHashPosCounter(HashList *Hash, HashPos *At)
 {
 	if ((Hash == NULL) || 
-	    (At->Position >= Hash->nMembersUsed) || 
+	    (At->Position >= Hash->nLookupTableItems) || 
 	    (At->Position < 0) ||
-	    (At->Position > Hash->nMembersUsed))
+	    (At->Position > Hash->nLookupTableItems))
 		return 0;
 	return At->Position;
 }
@@ -615,6 +716,7 @@ void DeleteHashPos(HashPos **DelMe)
 /**
  * @brief Get the data located where HashPos Iterator points at, and Move HashPos one forward
  * @param Hash your Hashlist to follow
+ * @param At the position to retrieve the Item from and move forward afterwards
  * @param HKLen returns Length of Hashkey Returned
  * @param HashKey returns the Hashkey corrosponding to HashPos
  * @param Data returns the Data found at HashPos
@@ -625,9 +727,9 @@ int GetNextHashPos(HashList *Hash, HashPos *At, long *HKLen, const char **HashKe
 	long PayloadPos;
 
 	if ((Hash == NULL) || 
-	    (At->Position >= Hash->nMembersUsed) || 
+	    (At->Position >= Hash->nLookupTableItems) || 
 	    (At->Position < 0) ||
-	    (At->Position > Hash->nMembersUsed))
+	    (At->Position > Hash->nLookupTableItems))
 		return 0;
 	*HKLen = Hash->LookupTable[At->Position]->HKLen;
 	*HashKey = Hash->LookupTable[At->Position]->HashKey;
@@ -643,6 +745,57 @@ int GetNextHashPos(HashList *Hash, HashPos *At, long *HKLen, const char **HashKe
 	return 1;
 }
 
+/**
+ * @brief Get the data located where HashPos Iterator points at
+ * @param Hash your Hashlist to follow
+ * @param At the position retrieve the data from
+ * @param HKLen returns Length of Hashkey Returned
+ * @param HashKey returns the Hashkey corrosponding to HashPos
+ * @param Data returns the Data found at HashPos
+ * \returns whether the item was found or not.
+ */
+int GetHashPos(HashList *Hash, HashPos *At, long *HKLen, const char **HashKey, void **Data)
+{
+	long PayloadPos;
+
+	if ((Hash == NULL) || 
+	    (At->Position >= Hash->nLookupTableItems) || 
+	    (At->Position < 0) ||
+	    (At->Position > Hash->nLookupTableItems))
+		return 0;
+	*HKLen = Hash->LookupTable[At->Position]->HKLen;
+	*HashKey = Hash->LookupTable[At->Position]->HashKey;
+	PayloadPos = Hash->LookupTable[At->Position]->Position;
+	*Data = Hash->Members[PayloadPos]->Data;
+
+	return 1;
+}
+
+/**
+ * @brief Move HashPos one forward
+ * @param Hash your Hashlist to follow
+ * @param At the position to move forward
+ * \returns whether there is a next item or not.
+ */
+int NextHashPos(HashList *Hash, HashPos *At)
+{
+	if ((Hash == NULL) || 
+	    (At->Position >= Hash->nLookupTableItems) || 
+	    (At->Position < 0) ||
+	    (At->Position > Hash->nLookupTableItems))
+		return 0;
+
+	/* Position is NULL-Based, while Stepwidth is not... */
+	if ((At->Position % abs(At->StepWidth)) == 0)
+		At->Position += At->StepWidth;
+	else 
+		At->Position += ((At->Position) % abs(At->StepWidth)) * 
+			(At->StepWidth / abs(At->StepWidth));
+	return !((At->Position >= Hash->nLookupTableItems) || 
+		 (At->Position < 0) ||
+		 (At->Position > Hash->nLookupTableItems));
+}
+
 /**
  * @brief Get the data located where At points to
  * note: you should prefer iterator operations instead of using me.
@@ -658,7 +811,7 @@ int GetHashAt(HashList *Hash,long At, long *HKLen, const char **HashKey, void **
 
 	if ((Hash == NULL) || 
 	    (At < 0) || 
-	    (At > Hash->nMembersUsed))
+	    (At > Hash->nLookupTableItems))
 		return 0;
 	*HKLen = Hash->LookupTable[At]->HKLen;
 	*HashKey = Hash->LookupTable[At]->HashKey;
@@ -667,6 +820,28 @@ int GetHashAt(HashList *Hash,long At, long *HKLen, const char **HashKey, void **
 	return 1;
 }
 
+/**
+ * @brief Get the data located where At points to
+ * note: you should prefer iterator operations instead of using me.
+ * @param Hash your Hashlist peek from
+ * @param HKLen returns Length of Hashkey Returned
+ * @param HashKey returns the Hashkey corrosponding to HashPos
+ * @param Data returns the Data found at HashPos
+ * \returns whether the item was found or not.
+ */
+/*
+long GetHashIDAt(HashList *Hash,long At)
+{
+	if ((Hash == NULL) || 
+	    (At < 0) || 
+	    (At > Hash->nLookupTableItems))
+		return 0;
+
+	return Hash->LookupTable[At]->Key;
+}
+*/
+
+
 /**
  * @brief sorting function for sorting the Hash alphabeticaly by their strings
  * @param Key1 first item
@@ -719,9 +894,9 @@ static int SortByHashKeys(const void *Key1, const void* Key2)
  */
 void SortByHashKey(HashList *Hash, int Order)
 {
-	if (Hash->nMembersUsed < 2)
+	if (Hash->nLookupTableItems < 2)
 		return;
-	qsort(Hash->LookupTable, Hash->nMembersUsed, sizeof(HashKey*), 
+	qsort(Hash->LookupTable, Hash->nLookupTableItems, sizeof(HashKey*), 
 	      (Order)?SortByKeys:SortByKeysRev);
 	Hash->tainted = 1;
 }
@@ -734,9 +909,9 @@ void SortByHashKey(HashList *Hash, int Order)
 void SortByHashKeyStr(HashList *Hash)
 {
 	Hash->tainted = 0;
-	if (Hash->nMembersUsed < 2)
+	if (Hash->nLookupTableItems < 2)
 		return;
-	qsort(Hash->LookupTable, Hash->nMembersUsed, sizeof(HashKey*), SortByHashKeys);
+	qsort(Hash->LookupTable, Hash->nLookupTableItems, sizeof(HashKey*), SortByHashKeys);
 }
 
 
@@ -758,9 +933,9 @@ const void *GetSearchPayload(const void *HashVoid)
  */
 void SortByPayload(HashList *Hash, CompareFunc SortBy)
 {
-	if (Hash->nMembersUsed < 2)
+	if (Hash->nLookupTableItems < 2)
 		return;
-	qsort(Hash->LookupTable, Hash->nMembersUsed, sizeof(HashKey*), SortBy);
+	qsort(Hash->LookupTable, Hash->nLookupTableItems, sizeof(HashKey*), SortBy);
 	Hash->tainted = 1;
 }
 
@@ -780,15 +955,6 @@ void SortByPayload(HashList *Hash, CompareFunc SortBy)
  */
 
 
-/*
- * Generic function to free a pointer.  This can be used as a callback with the
- * hash table, even on systems where free() is defined as a macro or has had other
- * horrible things done to it.
- */
-void generic_free_handler(void *ptr) {
-	free(ptr);
-}
-
 /*
  * Generic function to free a reference.  
  * since a reference actualy isn't needed to be freed, do nothing.
