@@ -1,5 +1,5 @@
 /*
- * $Id: mime_parser.c 7375 2009-04-19 09:33:54Z dothebart $
+ * $Id: mime_parser.c 8506 2010-04-10 09:01:25Z dothebart $
  *
  * This is the MIME parser for Citadel.
  *
@@ -24,20 +24,42 @@
 #include "libcitadel.h"
 #include "libcitadellocal.h"
 
-void extract_key(char *target, char *source, long sourcelen, char *key, long keylen)
+void extract_key(char *target, char *source, long sourcelen, char *key, long keylen, char KeyEnd)
 {
-	char *ptr;
+	char *sptr, *ptr = NULL;
 	int double_quotes = 0;
+	long RealKeyLen = keylen;
 
-	ptr = bmstrcasestr_len(source, sourcelen, key, keylen);
+	sptr = source;
 
+	while (sptr != NULL)
+	{
+		ptr = bmstrcasestr_len(sptr, sourcelen - (sptr - source), 
+				       key, keylen);
+		if(ptr != NULL)
+		{
+			while (isspace(*(ptr + RealKeyLen)))
+				RealKeyLen ++;
+			if (*(ptr + RealKeyLen) == KeyEnd)
+			{
+				sptr = NULL;
+				RealKeyLen ++;				
+			}
+			else
+			{
+				sptr = ptr + RealKeyLen + 1;
+			}
+		}
+		else 
+			sptr = ptr;
+	}
 	if (ptr == NULL) {
-		strcpy(target, "");
+		*target = '\0';
 		return;
 	}
-	strcpy(target, (ptr + keylen));
+	strcpy(target, (ptr + RealKeyLen));
 
-	for (ptr=target; (*ptr != 0); ++ptr) {
+	for (ptr=target; (*ptr != 0); ptr++) {
 
 		/* A semicolon means we've hit the end of the key, unless we're inside double quotes */
 		if ( (double_quotes != 1) && (*ptr == ';')) {
@@ -152,45 +174,11 @@ void mime_decode(char *partnum,
 		 char *disposition,
 		 char *id,
 		 char *name, char *filename,
-		 void (*CallBack)
-		  (char *cbname,
-		   char *cbfilename,
-		   char *cbpartnum,
-		   char *cbdisp,
-		   void *cbcontent,
-		   char *cbtype,
-		   char *cbcharset,
-		   size_t cblength,
-		   char *cbencoding,
-		   char *cbid,
-		   void *cbuserdata),
-		 void (*PreMultiPartCallBack)
-		  (char *cbname,
-		   char *cbfilename,
-		   char *cbpartnum,
-		   char *cbdisp,
-		   void *cbcontent,
-		   char *cbtype,
-		   char *cbcharset,
-		   size_t cblength,
-		   char *cbencoding,
-		   char *cbid,
-		   void *cbuserdata),
-		 void (*PostMultiPartCallBack)
-		  (char *cbname,
-		   char *cbfilename,
-		   char *cbpartnum,
-		   char *cbdisp,
-		   void *cbcontent,
-		   char *cbtype,
-		   char *cbcharset,
-		   size_t cblength,
-		   char *cbencoding,
-		   char *cbid,
-		   void *cbuserdata),
-		  void *userdata,
-		  int dont_decode
-)
+		 MimeParserCallBackType CallBack,
+		 MimeParserCallBackType PreMultiPartCallBack,
+		 MimeParserCallBackType PostMultiPartCallBack,
+		 void *userdata,
+		 int dont_decode)
 {
 
 	char *decoded;
@@ -255,45 +243,11 @@ void mime_decode(char *partnum,
  */
 void the_mime_parser(char *partnum,
 		     char *content_start, char *content_end,
-		     void (*CallBack)
-		      (char *cbname,
-		       char *cbfilename,
-		       char *cbpartnum,
-		       char *cbdisp,
-		       void *cbcontent,
-		       char *cbtype,
-		       char *cbcharset,
-		       size_t cblength,
-		       char *cbencoding,
-		       char *cbid,
-		       void *cbuserdata),
-		     void (*PreMultiPartCallBack)
-		      (char *cbname,
-		       char *cbfilename,
-		       char *cbpartnum,
-		       char *cbdisp,
-		       void *cbcontent,
-		       char *cbtype,
-		       char *cbcharset,
-		       size_t cblength,
-		       char *cbencoding,
-		       char *cbid,
-		       void *cbuserdata),
-		     void (*PostMultiPartCallBack)
-		      (char *cbname,
-		       char *cbfilename,
-		       char *cbpartnum,
-		       char *cbdisp,
-		       void *cbcontent,
-		       char *cbtype,
-		       char *cbcharset,
-		       size_t cblength,
-		       char *cbencoding,
-		       char *cbid,
-		       void *cbuserdata),
-		      void *userdata,
-		      int dont_decode
-)
+		     MimeParserCallBackType CallBack,
+		     MimeParserCallBackType PreMultiPartCallBack,
+		     MimeParserCallBackType PostMultiPartCallBack,
+		     void *userdata,
+		     int dont_decode)
 {
 
 	char *ptr;
@@ -392,9 +346,9 @@ void the_mime_parser(char *partnum,
 				memcpy (content_type, &header[13], headerlen - 12);
 				content_type_len = striplt (content_type);
 
-				extract_key(content_type_name, content_type, content_type_len, HKEY("name="));
-				extract_key(charset,           content_type, content_type_len, HKEY("charset="));
-				extract_key(boundary,          header,       headerlen,        HKEY("boundary="));
+				extract_key(content_type_name, content_type, content_type_len, HKEY("name"), '=');
+				extract_key(charset,           content_type, content_type_len, HKEY("charset"), '=');
+				extract_key(boundary,          header,       headerlen,        HKEY("boundary"), '=');
 
 				/* Deal with weird headers */
 				if (strchr(content_type, ' '))
@@ -405,8 +359,8 @@ void the_mime_parser(char *partnum,
 			else if (!strncasecmp(header, "Content-Disposition:", 20)) {
 				memcpy (disposition, &header[20], headerlen - 19);
 				disposition_len = striplt(disposition);
-				extract_key(content_disposition_name, disposition, disposition_len,  HKEY("name="));
-				extract_key(filename,                 disposition, disposition_len, HKEY("filename="));
+				extract_key(content_disposition_name, disposition, disposition_len,  HKEY("name"), '=');
+				extract_key(filename,                 disposition, disposition_len, HKEY("filename"), '=');
 			}
 			else if (!strncasecmp(header, "Content-ID:", 11)) {
 				strcpy(id, &header[11]);
@@ -631,50 +585,12 @@ end_parser:	/* free the buffers!  end the oppression!! */
  * considered to have ended when the parser encounters a 0x00 byte.
  */
 void mime_parser(char *content_start,
-		char *content_end,
-
-		 void (*CallBack)
-		  (char *cbname,
-		   char *cbfilename,
-		   char *cbpartnum,
-		   char *cbdisp,
-		   void *cbcontent,
-		   char *cbtype,
-		   char *cbcharset,
-		   size_t cblength,
-		   char *cbencoding,
-		   char *cbid,
-		   void *cbuserdata),
-
-		 void (*PreMultiPartCallBack)
-		  (char *cbname,
-		   char *cbfilename,
-		   char *cbpartnum,
-		   char *cbdisp,
-		   void *cbcontent,
-		   char *cbtype,
-		   char *cbcharset,
-		   size_t cblength,
-		   char *cbencoding,
-		   char *cbid,
-		   void *cbuserdata),
-
-		 void (*PostMultiPartCallBack)
-		  (char *cbname,
-		   char *cbfilename,
-		   char *cbpartnum,
-		   char *cbdisp,
-		   void *cbcontent,
-		   char *cbtype,
-		   char *cbcharset,
-		   size_t cblength,
-		   char *cbencoding,
-		   char *cbid,
-		   void *cbuserdata),
-
-		  void *userdata,
-		  int dont_decode
-)
+		 char *content_end,
+		 MimeParserCallBackType CallBack,
+		 MimeParserCallBackType PreMultiPartCallBack,
+		 MimeParserCallBackType PostMultiPartCallBack,
+		 void *userdata,
+		 int dont_decode)
 {
 
 	the_mime_parser("", content_start, content_end,
